@@ -2,7 +2,7 @@
  *
  *  Connection Manager
  *
- *  Copyright (C) 2007-2009  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2007-2010  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -37,67 +37,26 @@ static GSList *driver_list = NULL;
 
 struct connman_provider {
 	struct connman_element element;
+	struct connman_service *vpn_service;
 	char *identifier;
-	char *path;
-	enum connman_provider_state state;
-	enum connman_provider_error error;
 	char *name;
 	char *type;
 	char *dns;
 	char *domain;
-	DBusMessage *pending;
-	guint timeout;
 	struct connman_provider_driver *driver;
 	void *driver_data;
 };
 
-static const char *state2string(enum connman_provider_state state)
+void __connman_provider_append_properties(struct connman_provider *provider,
+							DBusMessageIter *iter)
 {
-	switch (state) {
-	case CONNMAN_PROVIDER_STATE_UNKNOWN:
-		break;
-	case CONNMAN_PROVIDER_STATE_IDLE:
-		return "idle";
-	case CONNMAN_PROVIDER_STATE_CONNECT:
-		return "connect";
-	case CONNMAN_PROVIDER_STATE_READY:
-		return "ready";
-	case CONNMAN_PROVIDER_STATE_DISCONNECT:
-		return "disconnect";
-	case CONNMAN_PROVIDER_STATE_FAILURE:
-		return "failure";
-	}
-	return NULL;
-}
+	if (provider->name != NULL)
+		connman_dbus_dict_append_basic(iter, "Name",
+					DBUS_TYPE_STRING, &provider->name);
 
-static const char *error2string(enum connman_provider_error error)
-{
-	switch (error) {
-	case CONNMAN_PROVIDER_ERROR_UNKNOWN:
-		break;
-	case CONNMAN_PROVIDER_ERROR_CONNECT_FAILED:
-		return "connect-failed";
-	}
-
-	return NULL;
-}
-
-static void append_path(gpointer key, gpointer value, gpointer user_data)
-{
-	struct connman_provider *provider = value;
-	DBusMessageIter *iter = user_data;
-
-	DBG("add provider path");
-	if (provider->path == NULL)
-		return;
-
-	dbus_message_iter_append_basic(iter, DBUS_TYPE_OBJECT_PATH,
-					&provider->path);
-}
-
-void __connman_provider_list(DBusMessageIter *iter, void *user_data)
-{
-	g_hash_table_foreach(provider_hash, append_path, iter);
+	if (provider->type != NULL)
+		connman_dbus_dict_append_basic(iter, "Type", DBUS_TYPE_STRING,
+						 &provider->type);
 }
 
 static struct connman_provider *connman_provider_lookup(const char *identifier)
@@ -109,17 +68,14 @@ static struct connman_provider *connman_provider_lookup(const char *identifier)
 	return provider;
 }
 
-static void connman_provider_setup_vpn_ipv4(struct connman_provider *provider,
+static int connman_provider_setup_vpn_ipv4(struct connman_provider *provider,
 						struct connman_element *element)
 {
 	if (element == NULL || provider == NULL)
-		return;
+		return -EINVAL;
 
 	DBG("set vpn type %d", element->type);
 
-	if (provider == NULL)
-		return;
-
 	g_free(element->ipv4.address);
 	element->ipv4.address = g_strdup(provider->element.ipv4.address);
 
@@ -132,11 +88,16 @@ static void connman_provider_setup_vpn_ipv4(struct connman_provider *provider,
 	g_free(element->ipv4.broadcast);
 	element->ipv4.broadcast = g_strdup(provider->element.ipv4.broadcast);
 
-	DBG("VPN exist");
+	g_free(element->ipv4.pac);
+	element->ipv4.pac = g_strdup(provider->element.ipv4.pac);
+
+	return connman_element_register(element, &provider->element);
 }
 
 struct connman_provider *connman_provider_ref(struct connman_provider *provider)
 {
+	DBG("provider %p", provider);
+
 	if (connman_element_ref(&provider->element) == NULL)
 		return NULL;
 
@@ -145,6 +106,8 @@ struct connman_provider *connman_provider_ref(struct connman_provider *provider)
 
 void connman_provider_unref(struct connman_provider *provider)
 {
+	DBG("provider %p", provider);
+
 	connman_element_unref(&provider->element);
 }
 
@@ -186,58 +149,19 @@ static int provider_probe(struct connman_provider *provider)
 	return 0;
 }
 
-static void state_changed(struct connman_provider *provider)
-{
-	const char *str;
-
-	str = state2string(provider->state);
-	if (str == NULL)
-		return;
-
-	connman_dbus_property_changed_basic(provider->path,
-				CONNMAN_PROVIDER_INTERFACE, "State",
-						DBUS_TYPE_STRING, &str); 
-}
-
-static void reply_pending(struct connman_provider *provider, int error)
-{
-	if (provider->timeout > 0) {
-		g_source_remove(provider->timeout);
-		provider->timeout = 0;
-	}
-
-	if (provider->pending != NULL) {
-		if (error > 0) {
-			DBusMessage *reply;
-
-			reply = __connman_error_failed(provider->pending,
-								error);
-			if (reply != NULL)
-				g_dbus_send_message(connection, reply);
-		} else
-			g_dbus_send_reply(connection, provider->pending,
-							DBUS_TYPE_INVALID);
-
-		dbus_message_unref(provider->pending);
-		provider->pending = NULL;
-	}
-}
-
-static int connman_provider_disconnect(struct connman_provider *provider)
+int __connman_provider_disconnect(struct connman_provider *provider)
 {
 	int err;
 
 	DBG("provider %p", provider);
 
-	reply_pending(provider, ECONNABORTED);
-
 	if (provider->driver != NULL && provider->driver->disconnect != NULL)
 		err = provider->driver->disconnect(provider);
 	else
 		return -EOPNOTSUPP;
 
-	__connman_provider_indicate_state(provider,
-					CONNMAN_PROVIDER_STATE_DISCONNECT);
+	__connman_service_indicate_state(provider->vpn_service,
+					CONNMAN_SERVICE_STATE_DISCONNECT);
 	if (err < 0) {
 		if (err != -EINPROGRESS)
 			return err;
@@ -248,123 +172,23 @@ static int connman_provider_disconnect(struct connman_provider *provider)
 	return 0;
 }
 
-int __connman_provider_indicate_state(struct connman_provider *provider,
-					enum connman_provider_state state)
-{
-	DBG("provider %p state %d", provider, state);
-
-	if (provider == NULL)
-		return -EINVAL;
-
-	if (provider->state == state)
-		return -EALREADY;
-
-	if (provider->state == CONNMAN_PROVIDER_STATE_FAILURE &&
-				state == CONNMAN_PROVIDER_STATE_IDLE)
-		return -EINVAL;
-
-	if (provider->state == CONNMAN_PROVIDER_STATE_IDLE &&
-				state == CONNMAN_PROVIDER_STATE_DISCONNECT)
-		return -EINVAL;
-
-	if (state == CONNMAN_PROVIDER_STATE_IDLE &&
-			provider->state != CONNMAN_PROVIDER_STATE_DISCONNECT) {
-		provider->state = CONNMAN_PROVIDER_STATE_DISCONNECT;
-		state_changed(provider);
-
-		connman_provider_disconnect(provider);
-	}
-
-	provider->state = state;
-	state_changed(provider);
-
-	if (state == CONNMAN_PROVIDER_STATE_READY)
-		reply_pending(provider, 0);
-
-	if (state == CONNMAN_PROVIDER_STATE_FAILURE)
-		reply_pending(provider, EIO);
-	else
-		provider->error = CONNMAN_PROVIDER_ERROR_UNKNOWN;
-
-	return 0;
-}
-
-int __connman_provider_indicate_error(struct connman_provider *provider,
-				      enum connman_provider_error error)
-{
-	DBG("provider %p error %d", provider, error);
-
-	if (provider == NULL)
-		return -EINVAL;
-
-	provider->error = error;
-
-	return __connman_provider_indicate_state(provider,
-					CONNMAN_PROVIDER_STATE_FAILURE);
-}
-
-static gboolean connect_timeout(gpointer user_data)
-{
-	struct connman_provider *provider = user_data;
-
-	DBG("provider %p", provider);
-
-	provider->timeout = 0;
-
-	if (provider->pending != NULL) {
-		DBusMessage *reply;
-
-		reply = __connman_error_operation_timeout(provider->pending);
-		if (reply != NULL)
-			g_dbus_send_message(connection, reply);
-
-		dbus_message_unref(provider->pending);
-		provider->pending = NULL;
-	}
-
-	__connman_provider_indicate_error(provider,
-				CONNMAN_PROVIDER_ERROR_CONNECT_FAILED);
-
-	return FALSE;
-}
-
-static connman_bool_t is_connecting(struct connman_provider *provider)
-{
-	switch (provider->state) {
-	case CONNMAN_PROVIDER_STATE_UNKNOWN:
-	case CONNMAN_PROVIDER_STATE_IDLE:
-	case CONNMAN_PROVIDER_STATE_FAILURE:
-	case CONNMAN_PROVIDER_STATE_DISCONNECT:
-	case CONNMAN_PROVIDER_STATE_READY:
-		break;
-	case CONNMAN_PROVIDER_STATE_CONNECT:
-		return TRUE;
-	}
-
-	return FALSE;
-}
-
-static int connman_provider_connect(struct connman_provider *provider)
+int __connman_provider_connect(struct connman_provider *provider)
 {
 	int err;
 
 	DBG("provider %p", provider);
 
-	if (provider->state == CONNMAN_PROVIDER_STATE_READY)
-		return -EISCONN;
-
-	if (is_connecting(provider) == TRUE)
-		return -EALREADY;
-
 	g_free(provider->element.ipv4.address);
 	g_free(provider->element.ipv4.netmask);
 	g_free(provider->element.ipv4.gateway);
 	g_free(provider->element.ipv4.broadcast);
+	g_free(provider->element.ipv4.pac);
 
 	provider->element.ipv4.address = NULL;
 	provider->element.ipv4.netmask = NULL;
 	provider->element.ipv4.gateway = NULL;
 	provider->element.ipv4.broadcast = NULL;
+	provider->element.ipv4.pac = NULL;
 
 	if (provider->driver != NULL && provider->driver->connect != NULL)
 		err = provider->driver->connect(provider);
@@ -375,11 +199,8 @@ static int connman_provider_connect(struct connman_provider *provider)
 		if (err != -EINPROGRESS)
 			return err;
 
-		provider->timeout = g_timeout_add_seconds(60,
-					connect_timeout, provider);
-
-		__connman_provider_indicate_state(provider,
-					CONNMAN_PROVIDER_STATE_CONNECT);
+		__connman_service_indicate_state(provider->vpn_service,
+					CONNMAN_SERVICE_STATE_ASSOCIATION);
 		return -EINPROGRESS;
 	}
 
@@ -403,141 +224,113 @@ int __connman_provider_remove(const char *path)
 	return 0;
 }
 
-static DBusMessage *get_properties(DBusConnection *conn,
-					DBusMessage *msg, void *user_data)
+static int set_connected(struct connman_provider *provider,
+					connman_bool_t connected)
 {
-	struct connman_provider *provider = user_data;
-	DBusMessage *reply;
-	DBusMessageIter array, dict;
-	dbus_bool_t required;
-	const char *str;
+	struct connman_service *service = provider->vpn_service;
 
-	DBG("provider %p", provider);
+	if (service == NULL)
+		return -ENODEV;
 
-	reply = dbus_message_new_method_return(msg);
-	if (reply == NULL)
-		return NULL;
-
-	dbus_message_iter_init_append(reply, &array);
-
-	connman_dbus_dict_open(&array, &dict);
-
-	if (provider->name != NULL)
-		connman_dbus_dict_append_basic(&dict, "Name",
-					DBUS_TYPE_STRING, &provider->name);
-
-	if (str != NULL)
-		connman_dbus_dict_append_basic(&dict, "Type",
-						 DBUS_TYPE_STRING,
-						 &provider->type);
-
-	str = state2string(provider->state);
-	if (str != NULL)
-		connman_dbus_dict_append_basic(&dict, "State",
-						 DBUS_TYPE_STRING, &str);
-
-	str = error2string(provider->error);
-	if (str != NULL)
-		connman_dbus_dict_append_basic(&dict, "Error",
-						 DBUS_TYPE_STRING, &str);
-
-	required = TRUE;
-	connman_dbus_dict_append_basic(&dict, "PassphraseRequired",
-					 DBUS_TYPE_BOOLEAN, &required);
-
-	connman_dbus_dict_close(&array, &dict);
-
-	return reply;
-}
-
-static GDBusMethodTable provider_methods[] = {
-	{ "GetProperties", "", "a{sv}", get_properties },
-	{ },
-};
-
-static GDBusSignalTable provider_signals[] = {
-	{ "PropertyChanged", "sv" },
-	{ },
-};
-
-int connman_provider_set_connected(struct connman_provider *provider,
-						connman_bool_t connected)
-{
 	if (connected == TRUE) {
 		enum connman_element_type type = CONNMAN_ELEMENT_TYPE_UNKNOWN;
 		struct connman_element *element;
+		char *nameservers = NULL, *name = NULL;
+		const char *value, *first;
+		int err;
+
+		__connman_service_indicate_state(provider->vpn_service,
+					CONNMAN_SERVICE_STATE_CONFIGURATION);
 
 		type = CONNMAN_ELEMENT_TYPE_IPV4;
 
 		element = connman_element_create(NULL);
-		if (element != NULL) {
-			element->type  = type;
-			element->index = provider->element.index;
+		if (element == NULL)
+			return -ENOMEM;
 
-			connman_provider_setup_vpn_ipv4(provider, element);
+		element->type  = type;
+		element->index = provider->element.index;
 
-			if (connman_element_register(element,
-					&provider->element) < 0)
-				connman_element_unref(element);
-			else {
-				char *nameservers = NULL;
-				const char *value;
-				char *name = NULL;
+		err = connman_provider_setup_vpn_ipv4(provider, element);
+		if (err < 0) {
+			connman_element_unref(element);
 
-				DBG("set dns");
-				nameservers = g_strdup(provider->dns);
-				value = nameservers;
-				name = connman_inet_ifname(
-						provider->element.index);
-				while (value) {
-					char *next = strchr(value, ' ');
-					if (next)
-						*(next++) = 0;
-
-					connman_resolver_append(name,
-							provider->domain,
-							value);
-					value = next;
-				}
-				DBG("free extra");
-				g_free(nameservers);
-				g_free(name);
-			}
+			__connman_service_indicate_state(service,
+						CONNMAN_SERVICE_STATE_FAILURE);
 
+			return err;
 		}
-		__connman_provider_indicate_state(provider,
-						  CONNMAN_PROVIDER_STATE_READY);
+
+		__connman_service_indicate_state(service,
+						CONNMAN_SERVICE_STATE_READY);
+
+		__connman_service_set_domainname(service, provider->domain);
+
+		nameservers = g_strdup(provider->dns);
+		value = nameservers;
+		first = strchr(value, ' ');
+		__connman_service_append_nameserver(service, first);
+		name = connman_inet_ifname(provider->element.index);
+		while (value) {
+			char *next = strchr(value, ' ');
+			if (next)
+				*(next++) = 0;
+
+			connman_resolver_append(name, provider->domain, value);
+			value = next;
+		}
+
+		g_free(nameservers);
+		g_free(name);
+
 	} else {
-		reply_pending(provider, ECONNABORTED);
 		connman_element_unregister_children(&provider->element);
-		__connman_provider_indicate_state(provider,
-					CONNMAN_PROVIDER_STATE_DISCONNECT);
+		__connman_service_indicate_state(service,
+						CONNMAN_SERVICE_STATE_IDLE);
 	}
 
 	return 0;
 }
 
+int connman_provider_set_state(struct connman_provider *provider,
+					enum connman_provider_state state)
+{
+	if (provider == NULL || provider->vpn_service == NULL)
+		return -EINVAL;
+
+	switch (state) {
+	case CONNMAN_PROVIDER_STATE_UNKNOWN:
+		return -EINVAL;
+	case CONNMAN_PROVIDER_STATE_IDLE:
+		return set_connected(provider, FALSE);
+	case CONNMAN_PROVIDER_STATE_CONNECT:
+		return __connman_service_indicate_state(provider->vpn_service,
+					CONNMAN_SERVICE_STATE_ASSOCIATION);
+	case CONNMAN_PROVIDER_STATE_READY:
+		return set_connected(provider, TRUE);
+	case CONNMAN_PROVIDER_STATE_DISCONNECT:
+		return __connman_service_indicate_state(provider->vpn_service,
+					CONNMAN_SERVICE_STATE_DISCONNECT);
+	case CONNMAN_PROVIDER_STATE_FAILURE:
+		return __connman_service_indicate_state(provider->vpn_service,
+					CONNMAN_SERVICE_STATE_FAILURE);
+	}
+
+	return -EINVAL;
+}
+
 static void provider_free(gpointer user_data)
 {
 	struct connman_provider *provider = user_data;
-	char *path = provider->path;
 
 	DBG("provider %p", provider);
 
-	reply_pending(provider, ENOENT);
-	provider->path = NULL;
-
-	if (path != NULL) {
-		g_dbus_unregister_interface(connection, path,
-						CONNMAN_PROVIDER_INTERFACE);
-		g_free(path);
-	}
-
 	g_free(provider->name);
 	g_free(provider->type);
 	g_free(provider->domain);
 	g_free(provider->identifier);
 	g_free(provider->dns);
+	__connman_service_put(provider->vpn_service);
 }
 
 static void unregister_provider(gpointer data)
@@ -546,7 +339,7 @@ static void unregister_provider(gpointer data)
 
 	DBG("provider %p", provider);
 
-	connman_provider_disconnect(provider);
+	__connman_provider_disconnect(provider);
 
 	connman_element_unregister(&provider->element);
 	connman_provider_unref(provider);
@@ -556,6 +349,8 @@ static void provider_destruct(struct connman_element *element)
 {
 	struct connman_provider *provider = element->private;
 
+	DBG("provider %p", provider);
+
 	provider_free(provider);
 }
 
@@ -563,8 +358,6 @@ static void __connman_provider_initialize(struct connman_provider *provider)
 {
 	DBG("provider %p", provider);
 
-	provider->state = CONNMAN_PROVIDER_STATE_UNKNOWN;
-
 	__connman_element_initialize(&provider->element);
 
 	provider->element.private = provider;
@@ -574,14 +367,13 @@ static void __connman_provider_initialize(struct connman_provider *provider)
 	provider->element.ipv4.netmask = NULL;
 	provider->element.ipv4.gateway = NULL;
 	provider->element.ipv4.broadcast = NULL;
+	provider->element.ipv4.pac = NULL;
 
 	provider->name = NULL;
 	provider->type = NULL;
 	provider->dns = NULL;
 	provider->domain = NULL;
 	provider->identifier = NULL;
-	provider->path = NULL;
-	provider->pending = NULL;
 }
 
 static struct connman_provider *connman_provider_new(void)
@@ -600,22 +392,8 @@ static struct connman_provider *connman_provider_new(void)
 
 static int provider_register(struct connman_provider *provider)
 {
-	const char *path = "/provider";
-
 	DBG("provider %p", provider);
 
-	if (provider->path != NULL)
-		return -EALREADY;
-
-	provider->path = g_strdup_printf("%s/%s", path, provider->identifier);
-
-	DBG("path %s", provider->path);
-
-	g_dbus_register_interface(connection, provider->path,
-					CONNMAN_PROVIDER_INTERFACE,
-					provider_methods, provider_signals,
-					NULL, provider, NULL);
-
 	return 0;
 }
 
@@ -652,9 +430,6 @@ static struct connman_provider *connman_provider_create(const char *name)
 	if (provider == NULL)
 		return NULL;
 
-	if (provider->path != NULL)
-		return provider;
-
 	provider_register(provider);
 
 	return provider;
@@ -664,7 +439,7 @@ int __connman_provider_create_and_connect(DBusMessage *msg)
 {
 	struct connman_provider *provider;
 	DBusMessageIter iter, array;
-	const char *type = NULL, *name = NULL;
+	const char *type = NULL, *name = NULL, *service_path = NULL;
 	char *ident;
 	gboolean created = FALSE;
 	int err;
@@ -755,12 +530,20 @@ int __connman_provider_create_and_connect(DBusMessage *msg)
 	if (created == TRUE)
 		provider_probe(provider);
 
-	err = connman_provider_connect(provider);
+	provider->vpn_service =
+			__connman_service_create_from_provider(provider);
+	if (provider->vpn_service == NULL) {
+		err = -EOPNOTSUPP;
+		goto failed;
+	}
+
+	err = __connman_service_connect(provider->vpn_service);
 	if (err < 0 && err != -EINPROGRESS)
 		goto failed;
 
+	service_path = __connman_service_get_path(provider->vpn_service);
 	g_dbus_send_reply(connection, msg,
-				DBUS_TYPE_OBJECT_PATH, &provider->path,
+				DBUS_TYPE_OBJECT_PATH, &service_path,
 							DBUS_TYPE_INVALID);
 	return 0;
 
@@ -768,11 +551,22 @@ failed:
 	if (provider != NULL && created == TRUE) {
 		DBG("can not connect delete provider");
 		connman_provider_unref(provider);
+
+		if (provider->vpn_service != NULL)
+			__connman_service_put(provider->vpn_service);
 	}
 
 	return err;
 }
 
+const char * __connman_provider_get_ident(struct connman_provider *provider)
+{
+	if (provider == NULL)
+		return NULL;
+
+	return provider->identifier;
+}
+
 int connman_provider_set_string(struct connman_provider *provider,
 					const char *key, const char *value)
 {
@@ -793,10 +587,13 @@ int connman_provider_set_string(struct connman_provider *provider,
 	} else if (g_str_equal(key, "Netmask") == TRUE) {
 		g_free(provider->element.ipv4.netmask);
 		provider->element.ipv4.netmask = g_strdup(value);
+	} else if (g_str_equal(key, "PAC") == TRUE) {
+		g_free(provider->element.ipv4.pac);
+		provider->element.ipv4.pac = g_strdup(value);
 	} else if (g_str_equal(key, "DNS") == TRUE) {
 		g_free(provider->dns);
 		provider->dns = g_strdup(value);
-	} else if (g_str_equal(key, "Domain")) {
+	} else if (g_str_equal(key, "Domain") == TRUE) {
 		g_free(provider->domain);
 		provider->domain = g_strdup(value);
 	}
@@ -829,6 +626,28 @@ void connman_provider_set_data(struct connman_provider *provider, void *data)
 
 void connman_provider_set_index(struct connman_provider *provider, int index)
 {
+	struct connman_service *service = provider->vpn_service;
+	struct connman_ipconfig *ipconfig;
+
+	DBG("");
+
+	ipconfig = __connman_service_get_ipconfig(service);
+
+	if (ipconfig == NULL) {
+		__connman_service_create_ipconfig(service, index);
+
+		ipconfig = __connman_service_get_ipconfig(service);
+		if (ipconfig == NULL) {
+			DBG("Couldnt create ipconfig");
+			goto done;
+		}
+	}
+
+	connman_ipconfig_set_method(ipconfig, CONNMAN_IPCONFIG_METHOD_FIXED);
+	__connman_ipconfig_set_index(ipconfig, index);
+
+
+done:
 	provider->element.index = index;
 }
 
